#ifdef GL_ES
precision highp  float;
#else
#define highp
#define mediump
#define lowp
#endif

uniform sampler2D inputImageTexture0; //原图

varying vec2 texcoordOut;

uniform float colorThreshold;
uniform vec3 dstColor;

vec3 rgbtohsv(vec3 rgb)
{
    float R = rgb.x;
    float G = rgb.y;
    float B = rgb.z;
    vec3 hsv = vec3(0.0,0.0,0.0);
    float max1 = max(R, max(G, B));
    float min1 = min(R, min(G, B));
    float m_delta = max1 - min1;

    hsv.z = max1/255.0;

    if(m_delta == 0.0){
        hsv.x = 0.0;
        hsv.y = 0.0;
    }
    else{
        if (R == max1)
        {
            hsv.x = (G - B) / m_delta;
        }
        if (G == max1)
        {
            hsv.x = 2.0 + (B - R) / m_delta;
        }
        if (B == max1)
        {
            hsv.x = 4.0 + (R - G) / m_delta;
        }

        hsv.x = hsv.x * 60.0;
        if (hsv.x  < 0.0)
        {
            hsv.x = hsv.x + 360.0;
        }

        hsv.y = m_delta / max1;
    }

    return hsv;
}

vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

mediump vec3 RGB2HSL(mediump vec3 rgb)
{
    mediump float D, Cmax, Cmin;
    mediump float H,S,L,R,G,B;
    
    R=rgb.r;
    G=rgb.g;
    B=rgb.b;
    // convert to HSL
    Cmax = max (R, max (G, B));
    Cmin = min (R, min (G, B));
    D = Cmax - Cmin;
    
    // calculate lightness
    L = (Cmax + Cmin) / 2.0;
    
    if (Cmax == Cmin) // it's gray
    {
        H = 0.0; // it's actually undefined
        S = 0.0;
    }
    else                        //Chromatic data...
    {
        if (L < 0.5) S = D / (Cmax + Cmin);
        else         S = D / (2.0 - Cmax - Cmin);
        
        mediump float del_R = (((Cmax - R) / 6.0) + (D / 2.0)) / D;
        mediump float del_G = (((Cmax - G) / 6.0) + (D / 2.0)) / D;
        mediump float del_B = (((Cmax - B) / 6.0) + (D / 2.0)) / D;
        
        if      (R == Cmax && G  >B) H = del_B - del_G;
        else if (R == Cmax && G  <= B) H = del_B - del_G + 1.0 ;
        else if (G == Cmax) H = (1.0 / 3.0) + del_R - del_B;
        else if (B == Cmax) H = (2.0 / 3.0) + del_G - del_R;
        
        if (H  < 0.000)  H += 1.0;
        if (H  >= 1.000)  H -= 1.0;
    }
    
    return vec3(H,S,L);
}

void main()
{
    vec4 srcColor = texture2D(inputImageTexture0, texcoordOut);

    vec3 orgHSL = RGB2HSL(srcColor.rgb);
    
    vec3 dstColorTemp = dstColor/255.0;
    vec3 dstHSL = RGB2HSL(dstColorTemp);
    
    mediump float vh;
    vh = orgHSL[0] - dstHSL[0];
    
    if(vh >= 0.5)
    {
        vh -= 1.0;
    }
    else{
        if (vh < -0.5){
            vh+=1.0;
        }
    }
    
    vh=abs(vh);
    float deltaJioaDu = colorThreshold/360.0;
    float alpha = 0.0;
    if (vh <= deltaJioaDu)
        alpha=1.0 - vh/deltaJioaDu;

    gl_FragColor = vec4(vec3(alpha),1.0);
}





