#ifdef GL_ES
precision highp  float;
#else
#define highp
#define mediump
#define lowp
#endif

uniform sampler2D inputImageTexture0; //原图

varying vec2 texcoordOut;
uniform float blendAlpha;

uniform vec3 dstColor;

const highp vec3 lumCoeff = vec3(0.2125, 0.7154, 0.0721);


mediump vec3 RGB2HSL(mediump vec3 rgb)
{
    mediump float D, Cmax, Cmin;
    mediump float H,S,L,R,G,B;
    
    R=rgb.r;
    G=rgb.g;
    B=rgb.b;
    // convert to HSL
    Cmax = max (R, max (G, B));
    Cmin = min (R, min (G, B));
    D = Cmax - Cmin;
    
    // calculate lightness
    L = (Cmax + Cmin) / 2.0;
    
    if (Cmax == Cmin) // it's gray
    {
        H = 0.0; // it's actually undefined
        S = 0.0;
    }
    else                        //Chromatic data...
    {
        if (L < 0.5) S = D / (Cmax + Cmin);
        else         S = D / (2.0 - Cmax - Cmin);
        
        mediump float del_R = (((Cmax - R) / 6.0) + (D / 2.0)) / D;
        mediump float del_G = (((Cmax - G) / 6.0) + (D / 2.0)) / D;
        mediump float del_B = (((Cmax - B) / 6.0) + (D / 2.0)) / D;
        
        if      (R == Cmax && G  >B) H = del_B - del_G;
        else if (R == Cmax && G  <= B) H = del_B - del_G + 1.0 ;
        else if (G == Cmax) H = (1.0 / 3.0) + del_R - del_B;
        else if (B == Cmax) H = (2.0 / 3.0) + del_G - del_R;
        
        if (H  < 0.000)  H += 1.0;
        if (H  >= 1.000)  H -= 1.0;
    }
    
    return vec3(H,S,L);
}

void main()
{
    vec4 srcColor = texture2D(inputImageTexture0, texcoordOut);
    vec3 orgHSL = RGB2HSL(srcColor.rgb);
    
    vec3 dstColorTemp = vec3(255,128,0)/255.0;
//    vec3 dstHSL = RGB2HSL(dstColorTemp);
    vec3 dstHSL = vec3(0.000000,0.111100,1.200000);
    
    mediump float vh,alpha;
    mediump float standHue = dstHSL.x;
    mediump float deltaJioaDu = dstHSL.y;
    mediump float sat = dstHSL.z;
    
    vh=orgHSL.r-standHue;
    if (vh > 0.5){
        vh-=1.0;
    }
    else{
        if (vh < -0.5){
            vh+=1.0;
        }
    }
    
    vh=abs(vh);
    
    if (vh >=deltaJioaDu)
        alpha=1.0;
    else
        alpha=vh/deltaJioaDu;
    
    mediump vec3 lum=vec3(dot(srcColor.rgb,lumCoeff));
    mediump vec3 clB=mix(srcColor.rgb,lum,alpha);
    
    mediump vec3 intensity = vec3(dot(clB,lumCoeff));
    clB = mix(intensity,clB,sat);
    vec4 effectColor = vec4(clB,srcColor.a);
    gl_FragColor = vec4(mix(srcColor,effectColor,blendAlpha).rgb,1.0);
}
