precision highp float;
 varying highp vec2 texcoordOut;
 
 uniform sampler2D inputImageTexture0;
 uniform sampler2D inputImageTexture1;
 uniform sampler2D inputImageTexture2;
 uniform float blurOpacity;
 uniform float skinOpacity;
 uniform float whiteOpacity;
 
 lowp float factor1 = 2.782;
 lowp float factor2 = 1.131;
 lowp float factor3 = 1.158;
 lowp float factor4 = 2.901;
 lowp float factor5 = 1.0;
 lowp float factor6 = 0.639;
 lowp float factor7 = 0.963;

 lowp vec3 rgb2hsv(lowp vec3 c) {
     lowp vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
     highp vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
     highp vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
     highp float d = q.x - min(q.w, q.y);
     highp float e = 1.0e-10;
     lowp vec3 hsv = vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
     return hsv;
 }
 
 lowp vec3 ContrastSaturationBrightness(lowp vec3 color, lowp float brt, lowp float sat, lowp float con)
 {
     const lowp float AvgLumR = 0.5;
     const lowp float AvgLumG = 0.5;
     const lowp float AvgLumB = 0.5;
     
     const lowp vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);
     
     lowp vec3 AvgLumin = vec3(AvgLumR, AvgLumG, AvgLumB);
     lowp vec3 brtColor = color * brt;
     lowp vec3 intensity = vec3(dot(brtColor, LumCoeff));
     lowp vec3 satColor = mix(intensity, brtColor, sat);
     lowp vec3 conColor = mix(AvgLumin, satColor, con);
     
     return conColor;
 }

lowp vec4 lut3d(highp vec4 textureColor)
{
    mediump float blueColor = textureColor.b * 15.0;
    mediump vec2 quad1;
    quad1.y = max(min(4.0,floor(floor(blueColor) / 4.0)),0.0);
    quad1.x = max(min(4.0,floor(blueColor) - (quad1.y * 4.0)),0.0);
    
    mediump vec2 quad2;
    quad2.y = max(min(floor(ceil(blueColor) / 4.0),4.0),0.0);
    quad2.x = max(min(ceil(blueColor) - (quad2.y * 4.0),4.0),0.0);
    
    highp vec2 texPos1;
    texPos1.x = (quad1.x * 0.25) + 0.5/64.0 + ((0.25 - 1.0/64.0) * textureColor.r);
    texPos1.y = (quad1.y * 0.25) + 0.5/64.0 + ((0.25 - 1.0/64.0) * textureColor.g);
    
    highp vec2 texPos2;
    texPos2.x = (quad2.x * 0.25) + 0.5/64.0 + ((0.25 - 1.0/64.0) * textureColor.r);
    texPos2.y = (quad2.y * 0.25) + 0.5/64.0 + ((0.25 - 1.0/64.0) * textureColor.g);
    
    lowp vec4 newColor1 = texture2D(inputImageTexture2, texPos1);
    lowp vec4 newColor2 = texture2D(inputImageTexture2, texPos2);
    
    mediump vec4 newColor = mix(newColor1, newColor2, fract(blueColor));
    return newColor;
}
 
 void main() {
    
     lowp vec4 inputColor = texture2D(inputImageTexture0, texcoordOut);
     lowp vec3 hsv = rgb2hsv(inputColor.rgb);
     
     lowp float opacityLimit = 1.0;

     if ((0.18 <= hsv.x && hsv.x <= 0.89) || hsv.z <= 0.2 || hsv.z >=0.99) {
         opacityLimit = 0.0;
     }
     if (0.16 < hsv.x && hsv.x < 0.18) {
         opacityLimit = min(opacityLimit, (0.18 - hsv.x) / 0.02);
     }
     if (0.89 < hsv.x && hsv.x < 0.91) {
         opacityLimit = min(opacityLimit, 1.0 - (0.91 - hsv.x) / 0.02);
     }
     if (0.2 < hsv.z && hsv.x < 0.3) {
         opacityLimit = min(opacityLimit, 1.0 - (0.3 - hsv.z) / 0.1);
     } 
     
     vec4 resultColor = inputColor;
     if (opacityLimit != 0.0) {
         lowp vec4 blurColor = texture2D(inputImageTexture1, texcoordOut);
     
         opacityLimit = blurOpacity * opacityLimit;
     
         lowp float cDistance = distance(vec3(0.0, 0.0, 0.0), max(blurColor.rgb - inputColor.rgb, 0.0)) * factor1;
         lowp vec3 brightColor = ContrastSaturationBrightness(inputColor.rgb, factor2, 1.0, factor3);
         lowp vec3 mix11Color = mix(inputColor.rgb, brightColor.rgb, cDistance);
     
         lowp float dDistance = distance(vec3(0.0, 0.0, 0.0), max(inputColor.rgb-blurColor.rgb, 0.0)) * factor4;
         lowp vec3 darkColor = ContrastSaturationBrightness(inputColor.rgb, factor5, 1.0, factor6);
         lowp vec3 mix115Color = mix(mix11Color.rgb, darkColor.rgb, dDistance);
     
         lowp vec3 mix12Color;
     
         if (factor7 < 0.999)
         {
             lowp vec3 mix116Color = mix(inputColor.rgb, mix115Color.rgb, factor7);
             mix12Color = mix(mix116Color.rgb, blurColor.rgb, opacityLimit);
         }
         else
         {
             mix12Color = mix(mix115Color.rgb, blurColor.rgb, opacityLimit);
         }
     
         vec4 skinColor;
         if (skinOpacity < 0.999)
         {
             skinColor = vec4(mix(inputColor.rgb, mix12Color.rgb, skinOpacity), 1.0);
         }
         else
         {
             skinColor = vec4(mix12Color.rgb, 1.0);
         }
         inputColor = skinColor;
         resultColor = skinColor;
     }
     
     resultColor = lut3d(resultColor);
     gl_FragColor = mix(inputColor, resultColor, whiteOpacity);
 }
